import os
import json
import logging

__author__ = 'Carlo Mazzaferro<cmazzafe@ucsd.edu>'

# ------ # ------ # ------ # LOCAL FILES CONFIG # ------ # ------ # ------ #
ROOT_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), os.pardir, os.pardir) # This is your Project Root
DATA_DIR = os.path.join(ROOT_DIR, 'data')
CONFIG_PATH = os.path.join(ROOT_DIR, 'config/config')
TOOL_SPECIFIC_CONFIG = os.path.join(ROOT_DIR, 'config/tool_specific_config')
MODES_PATH = os.path.join(DATA_DIR, 'modes')
AWS_DIR = os.path.join(ROOT_DIR, 'awsCluster')
TEST_DIR = os.path.join(AWS_DIR, 'tests')
ANALISYS_STEPS = ["fastqc", "bwa-alignment", "post-alignment", "gatk-haplotype"]

with open(MODES_PATH) as f:
    MODES = json.load(f)

# MODELS_DIR = os.path.join(ROOT_DIR, 'models')
# RESUTLS_DIR = os.path.join(ROOT_DIR, 'reports')

# ------ # ------ # ------ # SERVER FILES CONFIG # ------ # ------ # ------ #

SERVER_WORKSPACE = '/shared/workspace/'
SOFTWARE_DIR = os.path.join(SERVER_WORKSPACE, 'software')
SERVER_DATA_DIR = os.path.join(SERVER_WORKSPACE, 'data_archive/')

# ------ # ------ # ------ # SOFTWARE TOOLS PATHS # ------ # ------ # ------ #

KALLISTO = os.path.join(SOFTWARE_DIR, 'kallisto_linux-v0.42.1/kallisto')
KALLISTO_INDEX = os.path.join(SOFTWARE_DIR, 'kallisto_linux-v0.42.1/kallisto_index/kallisto_index')
BT2_HOME = os.path.join(SOFTWARE_DIR, 'bowtie2-2.3.0-legacy')
ADAPTER = 'TGGAATTCTCGGGTGCCAAGG'
INDEX = 'Homo sapiens'
FASTQC_PATH = os.path.join(SOFTWARE_DIR, 'FastQC/fastqc')
TRIMMO_PATH = os.path.join(SOFTWARE_DIR, 'Trimmomatic-0.36/trimmomatic-0.36.jar')
CUTADAPT_PATH = os.path.join(SOFTWARE_DIR, 'cutadapt-1.12/bin/cutadapt')
ALL_GENOMES_FA_FILEPATH = os.path.join(SERVER_WORKSPACE, 'data/hairpin.fa')  # TODO: add this fie, can't be found in server
INDICES_DIR = os.path.join(SERVER_WORKSPACE, 'data/indices')


logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.INFO)


if __name__ == '__main__':
    import yaml

    with open(TOOL_SPECIFIC_CONFIG + "/BWA.yaml", 'r') as stream:
        try:
            print(yaml.load(stream))
        except yaml.YAMLError as exc:
            print(exc)


from pymongo import MongoClient


class Filters(object):

    """
    Class for variant filtering. DB and collection names can be passed directly to the class and will be used in each
    method.
    """

    def __init__(self, db_name, collection_name):

        self.collection_name = collection_name
        self.db_name = db_name

    def rare_cancer_variant(self):
        """
        Function for retrieving rare cancer variants.
        :return:
        """

        client = MongoClient()
        db = getattr(client, self.db_name)
        collection = getattr(db, self.collection_name)

        filtered = collection.find({"$and": [
                                   {"$or": [{"esp6500siv2_all": {"$lt": 0.05}}, {"esp6500siv2_all": {"$exists": False}}]},
                                   {"$or": [{"func_knowngene": "exonic"}, {"func_knowngene": "splicing"}]},
                                   {"exonicfunc_knowngene": {"$ne": "synonymous SNV"}},
                                   {"genotype.filter_passing_reads_count": {"$gte": 10}},
                                   {"cosmic70": {"$exists": True}},
                                   {"1000g2015aug_all": {"$lt": 0.1}}

        ]})

        filtered = list(filtered)
        print('Variants found that match rarity criteria: {}'.format(len(filtered)))
        return filtered

    def rare_disease_variant(self):
        client = MongoClient()

        db = getattr(client, self.db_name)
        collection = getattr(db, self.collection_name)

        filtered = collection.find({"$and": [
                                   {"$or": [{"esp6500siv2_all": {"$lt": 0.05}}, {"esp6500siv2_all": {"$exists": False}}]},
                                   {"$or": [{"func_knowngene": "exonic"}, {"func_knowngene": "splicing"}]},
                                   {"exonicfunc_knowngene": {"$ne": "synonymous SNV"}},
                                   {"genotype.filter_passing_reads_count": {"$gte": 10}},
                                   {"$or": [{"cosmic70": {"$exists": True}}, {"clinvar": {"$exists": True}}]},
                                   {"1000g2015aug_all": {"$lt": 0.1}}

       ]})

        filtered = list(filtered)
        print ('Variants found that match rarity criteria: {}'.format(len(filtered)))
        return filtered

    def rare_high_impact_variants(self):
        """Rare high impact (CADD) variant filter"""

        client = MongoClient()

        db = getattr(client, self.db_name)
        collection = getattr(db, self.collection_name)

        filtered = collection.find({"$and": [
                                   {"$or": [{"esp6500siv2_all": {"$lt": 0.05}}, {"esp6500siv2_all": {"$exists": False}}]},
                                   {"$or": [{"func_knowngene": "exonic"}, {"func_knowngene": "splicing"}]},
                                   {"exonicfunc_knowngene": {"$ne": "synonymous SNV"}},
                                   {"genotype.filter_passing_reads_count": {"$gte": 10}},
                                   {"$or": [{"cosmic70": {"$exists": True}}, {"clinvar": {"$exists": True}}]},
                                   {"1000g2015aug_all": {"$lt": 0.1}},
                                   {"cadd.phred": {"$gte": 15}}      #This is the change CADD Phred score >= 15

         ]})


        filtered = list(filtered)
        print('Variants found that match rarity criteria: {}'.format(len(filtered)))
        return filtered






import pandas
import vcf
from pymongo import MongoClient


class FileWriter(object):

    #TODO: Tests for new classes.
    def __init__(self, db_name, collection_name):

        self.collection_name = collection_name
        self.db_name = db_name
        self.client = MongoClient()
        self.db = getattr(self.client, self.db_name)
        self.collection = getattr(self.db, self.collection_name)

    def generate_unfiltered_annotated_csv(self, filepath):
        """
        :param filepath: filpath (including name of output file) to which the output will be written
        :return: annotated csv file
        """
        all_my_data = list(self.collection.find({}))
        df = pandas.DataFrame(all_my_data)

        with open(filepath, "w") as f:
            df.to_csv(f)

        return 'Finished writing annotated CSV file'

    def generate_unfiltered_annotated_vcf(self, vcf_input_path, vcf_output_path, info_out=True):
        """
        :param vcf_input_path: template vcf file (inital vcf from which a new one will be created)
        :param vcf_output_path: name and filepath to where new vcf file will be written
        :param info_out: if set to true (Default), will write all annotation data to INFO column, else, it won't.
        :return: writed vcf file to specified directory
        """

        all_my_data = list(self.collection.find({}))
        chr_vars = []
        location_vars_ant = []
        location_vars_pos = []

        for i in range(0, len(all_my_data)):
            if all_my_data[i]['chr'] == 'chrMT':
                chr_vars.append('chrM')
            else:
                chr_vars.append(all_my_data[i]['chr'])
            location_vars_ant.append(all_my_data[i]['start'] + 1)
            location_vars_pos.append(all_my_data[i]['start'] - 1)

        vcf_reader = vcf.Reader(filename=vcf_input_path)
        vcf_writer = vcf.Writer(open(vcf_output_path, 'w'), vcf_reader)

        for i in range(0, len(chr_vars)):
            for record in vcf_reader.fetch(chr_vars[i], location_vars_pos[i], location_vars_ant[i]):
                if info_out is True:
                    record.INFO.update(all_my_data[i])
                    vcf_writer.write_record(record)
                else:
                    vcf_writer.write_record(record)

        return 'Finished writing annotated VCF file'


    @staticmethod
    def generate_annotated_csv(list_dictionaries, filepath):
        """
        :param list_dictionaries: list of annotated variants in dictionary
        :param filepath: filpath (including name of output file) to which the output will be written
        :return: annotated csv file
        """
        df = pandas.DataFrame(list_dictionaries)
        with open(filepath, "w") as f:
            df.to_csv(f)

        return 'Finished writing annotated, filtered CSV file'

    @staticmethod
    def generate_annotated_vcf(joint_list, vcf_input_path, vcf_output_path, info_out=True):
        """
        :param vcf_input_path: template vcf file (initial vcf from which a new one will be created)
        :param vcf_output_path: name and filepath to where new vcf file will be written
        :param joint_list: list of dictionaties contatining annotations from ANNOVAR & myvariant
        :param info_out: if set to true (Default), will write all annotation data to INFO column, else, it won't.

        :return: None
        """
        chr_vars = []
        location_vars_ant = []
        location_vars_pos = []

        for i in range(0, len(joint_list)):
            if joint_list[i]['chr'] == 'chrMT':
                chr_vars.append('chrM')
            else:
                chr_vars.append(joint_list[i]['chr'])
            location_vars_ant.append(joint_list[i]['start'] + 1)
            location_vars_pos.append(joint_list[i]['start'] - 1)

        vcf_reader = vcf.Reader(filename=vcf_input_path)
        vcf_writer = vcf.Writer(open(vcf_output_path, 'w'), vcf_reader)

        for i in range(0, len(chr_vars)):
            for record in vcf_reader.fetch(chr_vars[i], location_vars_pos[i], location_vars_ant[i]):
                if info_out is True:
                    record.INFO.update(joint_list[i])
                    vcf_writer.write_record(record)
                else:
                    vcf_writer.write_record(record)

        return 'Finished writing annotated, filtered VCF file'

    def generate_pandas_df(self):

        client = MongoClient()
        db = getattr(client, self.db_name)
        collection = getattr(db, self.collection_name)
        all_my_data = list(collection.find({}))

        df = pandas.DataFrame(all_my_data)
        return df



from __future__ import division, print_function
import vcf
import myvariant
import csv
import re
import itertools
from pymongo import MongoClient
import VAPr.vcf_parsing as vvp


class VariantParsing(object):

    supported_build_vers = ['hg19', 'hg18', 'hg38']

    def __init__(self, vcf_file, project_data, annotated_file=None):

        self.chunksize = 950
        self.step = 0
        self.txt_file = annotated_file
        self.vcf_file = vcf_file
        self.hgvs = HgvsParser(self.vcf_file)
        self.csv_parsing = TxtParser(self.txt_file)
        self.collection = project_data['project_name']
        self.db = project_data['db_name']
        self.patient_id = project_data['patient_id']
        self.sample_id = project_data['sample_id']
        self._buffer_len = 50000
        self._last_round = False

    def annotate_and_save(self, build_ver='hg19', buffer=False):

        build_ver = self.check_ver(build_ver)
        if not self.txt_file:

            while self.hgvs.num_lines > self.step * self.chunksize:

                list_hgvs_ids = self.hgvs.get_variants_from_vcf(self.step)
                myvariants_variants = self.get_dict_myvariant(list_hgvs_ids)

                if len(myvariants_variants) < self.chunksize:
                    self._last_round = True

                if self._last_round:
                    return 'Done'
                else:
                    self.export(myvariants_variants)
                    self.step += 1

            return 'Done'

        else:

            if buffer:
                variant_buffer = []
                while self.csv_parsing.num_lines > self.step * self.chunksize:

                    list_hgvs_ids = self.hgvs.get_variants_from_vcf(self.step)
                    myvariants_variants = self.get_dict_myvariant(list_hgvs_ids)
                    offset = len(list_hgvs_ids) - self.chunksize
                    csv_variants = self.csv_parsing.open_and_parse_chunks(self.step, build_ver=build_ver, offset=offset)

                    merged_list = []
                    for i, _ in enumerate(myvariants_variants):
                        merged_list.append(self.merge_dict_lists(myvariants_variants[i], csv_variants[i]))

                    variant_buffer.extend(merged_list)
                    self.step += 1

                    if len(merged_list) < self.chunksize:
                        self._last_round = True

                    if (len(variant_buffer) > self._buffer_len) or self._last_round:
                        print('Parsing Buffer...')
                        self.export(variant_buffer)
                        variant_buffer = []

                        if self._last_round:
                            return 'Done2'

                return 'Done1'

            else:

                while self.csv_parsing.num_lines > self.step*self.chunksize:

                    list_hgvs_ids = self.hgvs.get_variants_from_vcf(self.step)
                    myvariants_variants = self.get_dict_myvariant(list_hgvs_ids)
                    offset = len(list_hgvs_ids) - self.chunksize
                    csv_variants = self.csv_parsing.open_and_parse_chunks(self.step, offset=offset)

                    merged_list = []
                    for i, _ in enumerate(myvariants_variants):
                        merged_list.append(self.merge_dict_lists(myvariants_variants[i], csv_variants[i]))

                    if len(merged_list) < self.chunksize:
                        self._last_round = True

                    if self._last_round:
                        return 'Done'
                    else:
                        self.export(merged_list)
                        self.step += 1

            return 'Done'

    def check_ver(self, build_ver):
        """ Checking user input validity for genome build version """

        if not build_ver:
            return 'hg19'  # Default genome build vesion

        if build_ver not in self.supported_build_vers:
            raise ValueError('Build version must not recognized. Supported builds are'
                             ' %s, %s, %s' % (self.supported_build_vers[0],
                                              self.supported_build_vers[1],
                                              self.supported_build_vers[2]))
        else:
            return build_ver

    def export(self, list_docs):
        """
        Export data do a MongoDB server
        :param list_docs: list of dictionaries containing variant information
        :return: null
        """
        client = MongoClient()
        db = getattr(client, self.db)
        collection = getattr(db, self.collection)
        collection.insert_many(list_docs, ordered=False)

    @staticmethod
    def merge_dict_lists(*dict_args):
        """
        Given any number of dicts, shallow copy and merge into a new dict,
        precedence goes to key value pairs in latter dicts.
        """
        result = {}
        for dictionary in dict_args:
            result.update(dictionary)
        return result

    def get_dict_myvariant(self, variant_list):
        """
        Function designated to place the queries on myvariant.info servers.

        :param variant_list: list of HGVS variant ID's. Usually retrived beforehand using the method
        get_variants_from_vcf
        from the class VariantParsing.
        :return: list of dictionaries. Each dictionary contains data about a single variant.
        """

        mv = myvariant.MyVariantInfo()
        # This will retrieve a list of dictionaries
        variant_data = mv.getvariants(variant_list, as_dataframe=False)
        variant_data = self.remove_id_key(variant_data)

        return variant_data

    def remove_id_key(self, variant_data):

        for dic in variant_data:
            dic['hgvs_id'] = dic.pop("_id", None)
            dic['hgvs_id'] = dic.pop("query", None)
            dic['patient_id'] = self.patient_id
            dic['sample_id'] = self.sample_id

        return variant_data


class HgvsParser(object):

    def __init__(self, vcf_file):

        self.vcf = vcf_file
        self.num_lines = sum(1 for _ in open(self.vcf))
        self.chunksize = 950

    def get_variants_from_vcf(self, step):
        """
        Retrieves variant names from a LARGE vcf file.
        :param step: ...
        :return: a list of variants formatted according to HGVS standards
        """
        list_ids = []
        reader = vcf.Reader(open(self.vcf, 'r'))

        for record in itertools.islice(reader, step * self.chunksize, (step + 1) * self.chunksize):
            if len(record.ALT) > 1:
                for alt in record.ALT:
                    list_ids.append(myvariant.format_hgvs(record.CHROM, record.POS,
                                                          record.REF, str(alt)))
            else:
                list_ids.append(myvariant.format_hgvs(record.CHROM, record.POS,
                                                      record.REF, str(record.ALT[0])))

        return self.complete_chromosome(list_ids)

    @staticmethod
    def complete_chromosome(expanded_list):
        for i in range(0, len(expanded_list)):
            if 'M' in expanded_list[i]:
                one = expanded_list[i].split(':')[0]
                two = expanded_list[i].split(':')[1]
                if 'MT' not in one:
                    one = 'chrMT'
                expanded_list[i] = one + ':' + two
        return expanded_list


class TxtParser(object):

    def __init__(self, txt_file):

        self.txt_file = txt_file
        self.num_lines = sum(1 for _ in open(self.txt_file))
        self.chunksize = 950
        self.offset = 0
        self.hg_19_columns = ['chr',
                              'start',
                              'end',
                              'ref',
                              'alt',
                              'func_knowngene',
                              'gene_knowngene',
                              'genedetail_knowngene',
                              'exonicfunc_knowngene',
                              'tfbsconssites',
                              'cytoband',
                              'genomicsuperdups',
                              '1000g2015aug_all',
                              'esp6500siv2_all',
                              'cosmic70',
                              'nci60',
                              'otherinfo']

        self.hg_18_columns = self.hg_19_columns

        self.hg_38_columns = ['chr',
                              'start',
                              'end',
                              'ref',
                              'alt',
                              'func_knowngene',
                              'gene_knowngene',
                              'genedetail_knowngene',
                              'exonicfunc_knowngene',
                              'cytoband',
                              'genomicsuperdups',
                              '1000g2015aug_all',
                              'esp6500siv2_all',
                              'cosmic70',
                              'nci60',
                              'otherinfo']

    def open_and_parse_chunks(self, step, build_ver=None, offset=0):

        listofdicts = []
        with open(self.txt_file, 'r') as txt:

            reader = csv.reader(txt, delimiter='\t')
            header = self._normalize_header(next(reader))

            for i in itertools.islice(reader, (step*self.chunksize) + self.offset,
                                      ((step+1)*self.chunksize) + offset + self.offset):

                sparse_dict = dict(zip(header[0:len(header)-1], i[0:len(header)-1]))
                sparse_dict['otherinfo'] = i[-2::]
                if build_ver == 'hg19' or 'hg18':
                    dict_filled = {k: sparse_dict[k] for k in self.hg_18_columns if sparse_dict[k] != '.'}
                else:
                    dict_filled = {k: sparse_dict[k] for k in self.hg_38_columns if sparse_dict[k] != '.'}

                modeled = AnnovarModels(dict_filled)
                listofdicts.append(modeled.final_dict)

            self.offset += offset

        return listofdicts

    @staticmethod
    def _normalize_header(header):
        normalized = []

        for item in header:
            normalized.append(item.lower().replace('.', '_'))

        return normalized


class AnnovarModels(object):

    def __init__(self, dictionary):

        self.dictionary = dictionary
        self.existing_keys = self.dictionary.keys()
        self.final_dict = self.process()

    def process(self):
        for key in self.dictionary.keys():

            if self.dictionary['chr'] == 'chrM':
                self.dictionary['chr'] = 'chrMT'

            if key in ['1000g2015aug_all', 'esp6500si_all', 'nci60']:
                self.dictionary[key] = float(self.dictionary[key])

            if key in ['start', 'end']:
                self.dictionary[key] = int(self.dictionary[key])

            if key == 'cytoband':
                cytoband_data = CytoBand(self.dictionary['cytoband'])
                self.dictionary['cytoband'] = cytoband_data.fill()

            if key in ['genomicsuperdups', 'tfbsconssites']:
                self.dictionary[key] = self.to_dict(key)

            if key == 'otherinfo':
                self.dictionary[key] = [i for i in self.dictionary[key] if i != '.']

        self.dictionary['genotype'] = self.parse_genotype()

        return self.dictionary

    def parse_genotype(self):

        parser = vvp.VCFGenotypeStrings()
        genotype_to_fill = parser.parse(self.dictionary['otherinfo'][0], self.dictionary['otherinfo'][1])
        gen_dic = {'genotype': genotype_to_fill.genotype,
                   'filter_passing_reads_count': [self._to_int(genotype_to_fill.filter_passing_reads_count)],
                   'genotype_likelihoods': [float(genotype_to_fill.genotype_likelihoods[0].likelihood_neg_exponent),
                                            float(genotype_to_fill.genotype_likelihoods[1].likelihood_neg_exponent),
                                            float(genotype_to_fill.genotype_likelihoods[2].likelihood_neg_exponent)],
                   'alleles': [self._to_int(genotype_to_fill.alleles[0].read_counts),
                               self._to_int(genotype_to_fill.alleles[1].read_counts)]
                   }

        return gen_dic

    @staticmethod
    def _to_int(val):
        try:
            val = int(val)
        except:
            pass
        return val

    def to_dict(self, key):
        as_dict = dict(item.split("=") for item in self.dictionary[key].split(";"))
        as_dict["Score"] = float(as_dict["Score"])
        return as_dict


class CytoBand(object):

    def __init__(self, cyto_band_name):

        self.letters = set('XY')
        self.name = cyto_band_name
        self.processed = self.fill()

    def fill(self):

        processed = {'Name': self.name}
        spliced = re.split('(\D+)', self.name)

        if any((c in self.letters) for c in self.name):
            processed['Chromosome'] = spliced[1][0]
            processed['Band'] = spliced[1][1]
        else:
            processed['Chromosome'] = int(spliced[0])
            processed['Band'] = spliced[1]

        processed['Region'] = spliced[2]

        if '.' in spliced:
            processed['Sub_Band'] = spliced[-1]
        return processed



from __future__ import division, print_function
import subprocess
import shlex
import os
import csv
import glob
import time
import datetime
from watchdog.events import FileSystemEventHandler
from watchdog.observers import Observer
from collections import OrderedDict


class AnnovarWrapper(object):

    """Run Annovar as subprocess"""

    supported_build_vers = ['hg19', 'hg18', 'hg38']

    def __init__(self, input_vcf, output_csv, annovar_path, build_ver=None):

        self.input = input_vcf
        self.output = output_csv + os.path.splitext(os.path.basename(self.input))[0] + '_annotated'
        self.output_csv_path = output_csv
        self.path = annovar_path
        self.buildver = self.check_ver(build_ver)
        self.down_dd = '-webfrom annovar'
        self.annovar_hosted = OrderedDict({'knownGene': True,
                                           'tfbsConsSites': False,
                                           'cytoBand': False,
                                           'targetScanS': False,
                                           'genomicSuperDups': False,
                                           'esp6500siv2_all': True,
                                           '1000g2015aug': True,
                                           'popfreq_all_20150413': True,
                                           'clinvar_20161128': True,
                                           'cosmic70': True,
                                           'nci60': True,
                                           'avdblist': True})

        self.dl_list_command = 'avdblist'
        self.manual_update = {'clinvar_20161128': [datetime.datetime(2016, 11, 28)],
                              '1000g2015aug':  [datetime.datetime(2016, 8, 30)],
                              'popfreq_all_20150413': [datetime.datetime(2015, 4, 13)]
                              }

        self.hg_18_databases = OrderedDict({'knownGene': 'g',
                                            'tfbsConsSites': 'r',
                                            'cytoBand': 'r',
                                            'targetScanS': 'r',
                                            'genomicSuperDups': 'r',
                                            'esp6500siv2_all': 'f',
                                            '1000g2015aug': 'f',
                                            'cosmic70': 'f',
                                            'nci60': 'f',
                                            })

        self.hg_19_databases = OrderedDict({'knownGene': 'g',  # Ok all builds
                                            'tfbsConsSites': 'r',  #
                                            'cytoBand': 'r',
                                            'targetScanS': 'r',
                                            'genomicSuperDups': 'r',
                                            'esp6500siv2_all': 'f',
                                            '1000g2015aug': 'f',
                                            'popfreq_all_20150413': 'f',
                                            'clinvar_20161128': 'f',
                                            'cosmic70': 'f',
                                            'nci60': 'f',
                                            })

        self.hg_38_databases = OrderedDict({'knownGene': 'g',
                                            'cytoBand': 'r',
                                            'genomicSuperDups': 'r',
                                            'esp6500siv2_all': 'f',
                                            '1000g2015aug': 'f',
                                            'clinvar_20161128': 'f',
                                            'cosmic70': 'f',
                                            'nci60': 'f',
                                            })

        self.databases = self.get_databases()
        self.annovar_command_str = self.build_annovar_command_str()

    def download_dbs(self, all_dbs=True, dbs=None):

        if len(os.listdir(self.path + 'humandb/')) > 0:
            files = glob.glob(self.path + 'humandb/*')
            for f in files:
                os.remove(f)

        list_commands = self.build_db_dl_command_str(all_dbs, dbs)
        for command in list_commands:
            args = shlex.split(command)
            subprocess.Popen(args, stdout=subprocess.PIPE)
            run_handler(self.path + 'humandb/', cmds=list_commands, download=True,  annovar_path=self.path)

        return 'Finished downloading databases to {}'.format(self.path + 'humandb/')

    def run_annovar(self):
        """
        files = glob.glob(self.output_csv_path + '*')
        for f in files:
            if os.path.basename(f).startswith(os.path.basename(self.input).split('.')[0]):
                os.remove(f)
        """

        args = shlex.split(self.annovar_command_str)
        subprocess.Popen(args, stdout=subprocess.PIPE)
        run_handler(self.output_csv_path, annovar_path=self.path)

        return 'Finished running ANNOVAR on {}'.format(self.input)

    def build_annovar_command_str(self):

        dbs = ",".join(list(self.databases.keys()))
        dbs_args = ",".join(list(self.databases.values()))

        if '1000g2015aug' in dbs:
            dbs = dbs.replace('1000g2015aug', '1000g2015aug_all')
        command = " ".join(['perl', os.path.join(self.path, 'table_annovar.pl'), self.input,
                            os.path.join(self.path, 'humandb/'), '-buildver', self.buildver, '-out',
                            self.output, '-remove -protocol', dbs,  '-operation',
                            dbs_args, '-nastring .', '-otherinfo -vcfinput'])

        return command

    def build_db_dl_command_str(self, all_dbs, dbs):

        if not all_dbs:
            for db in dbs:
                if db not in self.databases:
                    raise ValueError('Database %s not supported for buil version %s' % (db, self.buildver))
            self.databases = {db: self.databases[db] for db in dbs}

        command_list = []

        for db in self.databases:

            if self.annovar_hosted[db]:
                command_list.append(" ".join(['perl', self.path + 'annotate_variation.pl', '-build', self.buildver,
                                              '-downdb', self.down_dd, db, self.path + 'humandb/']))
            else:
                command_list.append(" ".join(['perl', self.path + 'annotate_variation.pl', '-build', self.buildver,
                                              '-downdb', db, self.path + 'humandb/']))
        return command_list

    def check_for_database_updates(self):

        self.download_dbs(all_dbs=False, dbs=['avdblist'])

        with open(os.path.join(self.path, '/humandb' + self.buildver + '_avdblist.txt'), 'r') as db_list:
            reader = csv.reader(db_list, delimiter='\t')
            db_dict = {}
            for i in reader:

                if i[0][-6:] != 'idx.gz':
                    db_dict[i[0][5:]] = [datetime.datetime(int(i[1][0:4]), int(i[1][4:6]), int(i[1][6:8])), i[2]]

        for db in self.manual_update.keys():
            for db_ in db_dict.keys():
                if db_.startswith(db):
                    if db_dict[db_][0] > self.manual_update[db][0]:
                        print('Database %s outdated, will download newer version' % db_)
                        self.download_dbs(all_dbs=False, dbs=[os.path.splitext(os.path.splitext(db_)[0])[0]])

    def check_ver(self, build_ver):

        if not build_ver:
            self.buildver = 'hg19'  # Default genome build vesion

        if build_ver not in self.supported_build_vers:
            raise ValueError('Build version must not recognized. Supported builds are'
                             ' %s, %s, %s' % (self.supported_build_vers[0],
                                              self.supported_build_vers[1],
                                              self.supported_build_vers[2]))
        else:
            self.buildver = build_ver

        return self.buildver

    def get_databases(self):

        if self.buildver == 'hg18':
            databases = self.hg_18_databases
        elif self.buildver == 'hg19':
            databases = self.hg_19_databases
        else:
            databases = self.hg_38_databases

        return databases


class MyHandler(FileSystemEventHandler):
    """
    Overwrite the methods for creation of files as annovar runs. Once the .csv file is detected, exit process
    and proceed to next file.
    """

    def __init__(self, observer, cmds=None,  download=False, annovar_path='ANNOVAR_PATH'):

        self.observer = observer
        self.dl = download
        self.annovar = annovar_path
        self.cmds = cmds

    def on_created(self, event):

        if self.dl:

            print('Downloading: ' + event.src_path)
            """
            dowloaded = ['annovar_date', '']

            if os.path.basename(event.src_path).split('.')[0] not in dowloaded:
                print("Currently downloading database file: " + os.path.basename(event.src_path).split('.')[0])

            dowloaded.append(os.path.basename(event.src_path).split('.')[0])

            final = event.src_path[-3:]
            if final == 'txt':
                print("\nAnnovar finished dowloading on file : " + os.path.basename(event.src_path).split('.')[0] +
                      ". A .txt file has been created in the %s directory\n" % self.annovar)
            """
            if self.cmds[-1][-2] in event.src_path:
                self.observer.stop()

        else:
            if event.src_path.split('.')[-1] not in ["invalid_input", "log", "avinput"]:
                print("Currently working on VCF file: " + event.src_path.split('/')[-1].split('.')[0] + ", field " +
                      event.src_path.split('/')[-1].split('.')[-1])

                final = event.src_path[-3:]
                if final == 'txt':
                    print("\nAnnovar finished working on file : " + event.src_path.split('/')[-1].split('.')[0] +
                          ". A .txt file has been created in the OUT_PATH directory\n")
                    self.observer.stop()


def run_handler(output_csv_path, cmds=None, download=False, annovar_path='ANNOVAR_PATH'):
    observer = Observer()
    event_handler = MyHandler(observer, cmds=cmds, download=download, annovar_path=annovar_path)
    observer.schedule(event_handler, output_csv_path, recursive=True)
    observer.start()
    observer.join()

    return None



from setuptools import setup

setup(name='VAPr',
      version='1.5',
      description='Package for NoSQL variant data storage, annotation and prioritization.',
      url='https://github.com/ucsd-ccbb/VAPr',
      author='Carlo Mazzaferro',
      author_email='cmazzafe@ucsd.edu',
      install_requires=['pymongo', 'myvariant', 'pyvcf', 'watchdog'],
      license='MIT',
      packages=['VAPr'],
      zip_safe=False)
